# bcodec - Ruby library to decode/encode bencoded data
# Copyright (C) 2006,2007 Thomas <tochiro+bcodecNO@SPAMrubyforge.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

module BCodec

  class InvalidInteger < Exception; end
  class InvalidString < Exception; end
  class InvalidKey < Exception; end
  class UnknownData < Exception; end
  class InvalidDataType < Exception; end

  #Decodes bencoded data from an IO-like object (IO, File, StringIO or similar)
  #
  #Returns the decoded object which is either of these:
  #Array, Integer (Fixnum or Bignum), Hash or String
  def decode(io)
    c = io.getc

    case c.chr
    when 'i'
      i = ''
      while c = io.getc
        if c.chr == 'e'
          if i.match(/^(0|-?[1-9][0-9]*)$/)
            return i.to_i
          else
            raise InvalidInteger
          end
        else
          i += c.chr
        end
      end
      raise EOFError
    when '0'..'9'
      n = c.chr
      while b = io.getc
        case b.chr
        when '0'..'9'
          n += b.chr
        when ':'
          strlen = n.to_i
          str = io.read(strlen)
          if str.length == strlen
            return str
          else
            raise EOFError
          end
        else
          raise InvalidString
        end
      end
    when 'l'
      list = []
      
      while item = decode(io)
        list.push(item)
      end

      return list
    when 'd'
      dict = {}

      while key = decode(io)
        raise InvalidKey unless key.instance_of?(String)
        val = decode(io)
        dict[key] = val
      end

      return dict
    when 'e'
      return false
    else
      raise UnknownData
    end
  end

  module_function :decode
  
end
