# bcodec - Ruby library to decode/encode bencoded data
# Copyright (C) 2006,2007 Thomas <tochiro+bcodecNO@SPAMrubyforge.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

module BCodec

  class InvalidKey < Exception; end
  class InvalidDataType < Exception; end

  #Encodes a Ruby object which must be either of these:
  #Array, Integer (Fixnum or Bignum), Hash or String
  #
  #Returns the encoded object(s) as a String
  def encode(obj)
    str = ""
    if obj.class == String
      str << "%d:%s" % [ obj.length, obj ]
    elsif obj.class == Fixnum or obj.class == Bignum
      str << "i" <<  obj.to_s << "e"
    elsif obj.class == Array
      str << "l"
      obj.each { |i| str += encode(i) }
      str << "e"
    elsif obj.class == Hash
      str << "d"
      obj.sort.each do |k, v|
        raise InvalidKey unless k.instance_of?(String)
        str << encode(k)
        str << encode(v)
      end
      str << "e"
    else
      raise InvalidDataType
    end

    return str
  end

  module_function :encode
  
end
