require 'stringio'
require 'test/unit'
require 'lib/bcodec/decode'

include BCodec

class TestDecoding < Test::Unit::TestCase

  def setup
    @tests = []
    @tests << [ "4:test", "test" ]
    @tests << [ "i666e", 666 ]
    @tests << [ "l4:testi666ee", [ "test", 666 ] ]
    @tests << [ "d4:testi666ee", { "test" => 666 } ]
    @tests << [ "d4:testli1ei2ei3ed1:Ai1e1:Bl1:a1:b1:ceeee", 
                { "test" => [ 1, 2, 3, { "A" => 1, "B" => [ 'a', 'b', 'c' ] } ] } ]

    @etests = []
    @etests << [ BCodec::InvalidInteger, "i1.0e" ]
    @etests << [ BCodec::InvalidInteger, "iAe" ]
    @etests << [ BCodec::UnknownData, "f" ]
    @etests << [ BCodec::InvalidKey, "di666e:3:keye" ]
    @etests << [ EOFError, "i1" ]
    @etests << [ EOFError, "4:ab" ]
  end

  def test_decoding
    @tests.each do |test, result|
      assert_equal decode(StringIO.new(test)), result
    end
  end
  
  def test_decoding_exceptions
    @etests.each do |exception, test|
      assert_raise(exception) { decode(StringIO.new(test)) }
    end
  end

end
