require 'stringio'
require 'test/unit'
require 'lib/bcodec/encode'

include BCodec

class TestEncoding < Test::Unit::TestCase

  def setup
    @tests = []
    @tests << [ "test", "4:test" ]
    @tests << [ 666, "i666e" ]
    @tests << [ [ "test", 666 ], "l4:testi666ee" ]
    @tests << [ { "test" => 666 }, "d4:testi666ee" ]
    @tests << [ { "test" => [ 1, 2, 3, { "A" => 1, "B" => [ 'a', 'b', 'c' ] } ] }, 
                "d4:testli1ei2ei3ed1:Ai1e1:Bl1:a1:b1:ceeee" ]

    @etests = []
    @etests << [ BCodec::InvalidDataType, :symbol ]
    @etests << [ BCodec::InvalidDataType, [1.0, "float"] ]
    @etests << [ BCodec::InvalidKey, {1 => "key"} ]
    @etests << [ BCodec::InvalidDataType, 1..666 ]
  end

  def test_basic_encoding
    @tests.each do |test, result|
      assert_equal(encode(test), result)
    end
  end

  def test_encoding_exceptions
    @etests.each do |exception, test|
      assert_raise(exception) { encode(test) }
    end
  end

end
